<!DOCTYPE html>
<html>
    <head>
        <title> - Dashboard</title>

        <!-- Bootstrap -->
        <link rel="stylesheet" href="/static_assets/css/bootstrap.min.css">
        <link rel="stylesheet" href="/static_assets/css/bootstrap-theme.min.css">
    </head>
    <body>
        <div class="container">
            {% block content %}{% end %}
        </div>
        <!-- Placed at the end of the document so the pages load faster -->
        <script src="/static_assets/js/jquery-2.1.0.min.js"></script>
        <!--script src="/static_assets/js/i18next.min.js"></script-->
        <script src="/static_assets/js/bootstrap.min.js"></script>
        <script src="/static_assets/js/can.custom.2.0.5.jquery.js"></script>
    </body>
</html>
